#
#       Whittaker_Smooth: function to perform Whittaker smoothing
#
#       Input: a = ages
#              d = deaths
#              e = central exposures
#              l = smoothing parameter, lambda
#
#       Output: graph of data with Whittaker smooth
#               Smooth = smoothed log(mortalities)
#
Whittaker_Smooth = function(a, d, e, l){
  Obs = log(d/e) 
  n = length(Age)
  Id = diag(n)
  D3 = diff(Id, diff = 3)
  Smooth = solve(Id + l*t(D3) %*% D3, Obs)
  plot(a + 0.5, Obs, xlab = "Age", ylab = "log(mortality)")
  lines(a + 0.5, Smooth, col = "red", lwd = 2)
  return(list(Smooth = Smooth))
}
#
#  Example
#
Data = read.table("UK_Male_2to30_2011.dat")
Age = Data[ ,1]
Dth = Data[ ,2]
Exp = Data[ ,3]
#
Output = Whittaker_Smooth(Age, Dth, Exp, 10)     #   lambda = 10
title(main = "Whittaker smoothing of UK male mortality in 2011")
Smooth = Output$Smooth
